/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFWGSTRENDERERDOLBY_H
#define MAFWGSTRENDERERDOLBY_H

#include <QObject>
#include <GConfItem>

/**
 * Provides volume setting, getting and listening functionality using
 * PulseAudioMainVolume DBus API.
 */
class MafwGstRendererDolby : public QObject
{
    Q_OBJECT
public:
    /**
     * Constructor
     */
    MafwGstRendererDolby( QObject* parent );
    ~MafwGstRendererDolby();
    void initialize();
    bool setMusicDolbyState (uint value);
    bool setMusicDolbyRoom (int value);
    bool setMusicDolbyColor (int value);
    uint getMusicDolbyState ();
    int getMusicDolbyRoom ();
    int getMusicDolbyColor ();
    bool setVideoDolbyState (uint value);
    bool setVideoDolbyRoom (int value);
    bool setVideoDolbyColor (int value);
    uint getVideoDolbyState ();
    int getVideoDolbyRoom ();
    int getVideoDolbyColor ();

Q_SIGNALS:
    /**
     * Signal telling that music surround is OFF/ON/AUTO.
     */
    void mafwDHMMusicPropertyChanged();
    /**
     * Signal telling that video surround is OFF/ON/AUTO.
     */
    void mafwDHMVideoPropertyChanged();

private Q_SLOTS:
    void valueMusicChanged();
    void valueVideoChanged();

private:
    uint m_currentMusicDolbyState;
    int m_currentMusicDolbyRoom;
    int m_currentMusicDolbyColor;
    uint m_currentVideoDolbyState;
    int m_currentVideoDolbyRoom;
    int m_currentVideoDolbyColor;

    GConfItem *m_dolbyConfMusic;
    GConfItem *m_dolbyConfMusicRoom;
    GConfItem *m_dolbyConfMusicColor;
    GConfItem *m_dolbyConfVideo;
    GConfItem *m_dolbyConfVideoRoom;
    GConfItem *m_dolbyConfVideoColor;
};

#endif // MAFWGSTRENDERERDOLBY_H
