/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#include "QNetworkStubs.h"

#include <QDebug>
#include <QTimer>
#include <QtNetwork/QNetworkConfiguration>
#include <QtNetwork/QNetworkConfigurationManager>


NetworkStubHelper networkStub;
int NetworkStubHelper::networkConfigurationManagerInstances = 0;

/**

  QNetworkConfiguration

  **/

bool NetworkStubHelper::currentCreatedConfigIsValid = false;

static int staticCreationId = 0;
class QNetworkConfigurationPrivate : public QSharedData
{
public:
    int creationId;
    bool isValid;
};

QNetworkConfiguration::QNetworkConfiguration()
{
    d = new QNetworkConfigurationPrivate;
    d->creationId = ++staticCreationId;
    d->isValid = networkStub.currentCreatedConfigIsValid;
}

QNetworkConfiguration::~QNetworkConfiguration()
{

}

QNetworkConfiguration::QNetworkConfiguration(const QNetworkConfiguration &other)
{
    d = new QNetworkConfigurationPrivate;
    d->creationId = other.d->creationId;
}

QNetworkConfiguration::StateFlags QNetworkConfiguration::state() const
{
    return networkStub.m_currentConfState;
}

QString QNetworkConfiguration::name() const
{
    return "Test conf";
}

bool QNetworkConfiguration::isValid() const
{
    return d->isValid;
}

QNetworkConfiguration& QNetworkConfiguration::operator=(QNetworkConfiguration const &other)
{
    d->creationId = other.d->creationId;
    return *this;
}

bool QNetworkConfiguration::operator==(const QNetworkConfiguration &other ) const
{
    return d->creationId == other.d->creationId;
}

/**

  QNetworkConfigurationManager

  **/

QNetworkConfigurationManager::QNetworkConfigurationManager(QObject *parent)
    :
    QObject(parent)
{
    ++networkStub.networkConfigurationManagerInstances;
    if( networkStub.networkConfigurationManagerInstances == 1)
    {
        connect( &networkStub, SIGNAL(configurationChange(QNetworkConfiguration)),
                     this, SIGNAL(configurationChanged(QNetworkConfiguration)));
    }
}

QNetworkConfigurationManager::~QNetworkConfigurationManager()
{
    --networkStub.networkConfigurationManagerInstances;
    if( networkStub.networkConfigurationManagerInstances == 0 )
    {
        networkStub.disconnect(this);
    }
}
