/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */
#include "ut_MafwGstRendererNetworkMonitor.h"
#include "MafwGstRendererNetworkMonitor.h"
#include "QNetworkStubs.h"

#include <QSignalSpy>
#include <QDebug>

extern NetworkStubHelper networkStub;

void Ut_MafwGstRendererNetworkMonitor::initTestCase()
{

}

void Ut_MafwGstRendererNetworkMonitor::cleanupTestCase()
{

}

void Ut_MafwGstRendererNetworkMonitor::testPrepareNetworkChangeSignal()
{
    MafwGstRendererNetworkMonitor monitor;
    QNetworkConfiguration config;

    QSignalSpy networkChangeFinished(&monitor, SIGNAL(networkChangeFinished()));

    networkStub.emitConfigurationChange(config, QNetworkConfiguration::Active);

    QCOMPARE(networkChangeFinished.size(), 1);
}

void Ut_MafwGstRendererNetworkMonitor::testOfflineSignal()
{
    networkStub.currentCreatedConfigIsValid = false;
    MafwGstRendererNetworkMonitor monitor;
    networkStub.currentCreatedConfigIsValid = true;

    QSignalSpy prepareNetworkChangeSpy(&monitor, SIGNAL(prepareNetworkChange()));

    QNetworkConfiguration config;
    networkStub.emitConfigurationChange(config, QNetworkConfiguration::Discovered);

    QCOMPARE(prepareNetworkChangeSpy.size(), 1);

    //first activate
    networkStub.emitConfigurationChange(config, QNetworkConfiguration::Active);
    //then set it not active
    networkStub.emitConfigurationChange(config, QNetworkConfiguration::Discovered);

    QCOMPARE(prepareNetworkChangeSpy.size(), 2);

}

void Ut_MafwGstRendererNetworkMonitor::testConfigChanges()
{
    MafwGstRendererNetworkMonitor monitor;
    QSignalSpy networkChangeSpy(&monitor, SIGNAL(prepareNetworkChange()));
    QSignalSpy networkChangeFinished(&monitor, SIGNAL(networkChangeFinished()));

    QNetworkConfiguration config;
    networkStub.emitConfigurationChange(config, QNetworkConfiguration::Active);
    QNetworkConfiguration config2;
    networkStub.emitConfigurationChange(config2, QNetworkConfiguration::Active);
    networkStub.emitConfigurationChange(config, QNetworkConfiguration::Discovered);

    QCOMPARE(networkChangeFinished.size(), 2);
    QCOMPARE(networkChangeSpy.size(), 0);

    networkStub.emitConfigurationChange(config2, QNetworkConfiguration::Discovered);
    QCOMPARE(networkChangeSpy.size(), 1);
}

QTEST_MAIN(Ut_MafwGstRendererNetworkMonitor)

